@@ -5,8 +5,9 @@
 #'
 #' This set of functions can be used to create new vector datasets,
 #' test existence of dataset/layer/field, test dataset and layer capabilities,
-#' create new layers in an existing dataset, delete layers, create new
-#' attribute and geometry fields on an existing layer, rename and delete
+#' create new layers in an existing dataset, get the names of field domains
+#' stored in a dataset, write field domains in a dataset, delete layers, create
+#' new attribute and geometry fields on an existing layer, rename and delete
 #' fields, and edit data with SQL statements.
 #'
 #' @name ogr_manage
@@ -18,7 +19,7 @@
 #'
 #' `ogr_ds_exists()` tests whether a vector dataset can be opened from the
 #' given data source name (DSN), potentially testing for update access.
-#' Returns a logical scalar.
+#' Returns a logical value.
 #'
 #' `ogr_ds_format()` returns a character string containing the short name of
 #' the format driver for a given DSN, or `NULL` if the dataset cannot be
@@ -30,22 +31,28 @@
 #' returned if `dsn` cannot be opened with the requested access.
 #' Wrapper of `GDALDatasetTestCapability()` in the GDAL API.
 #' The returned list contains the following named elements:
-#' * `CreateLayer`: `TRUE` if this datasource can create new layers
-#' * `DeleteLayer`: `TRUE` if this datasource can delete existing layers
+#' * `CreateLayer`: `TRUE` if this dataset can create new layers
+#' * `DeleteLayer`: `TRUE` if this dataset can delete existing layers
 #' * `CreateGeomFieldAfterCreateLayer`: `TRUE` if the layers of this
-#' datasource support geometry field creation just after layer creation
-#' * `CurveGeometries`: `TRUE` if this datasource supports curve geometries
-#' * `Transactions`: `TRUE` if this datasource supports (efficient)
+#' dataset supports geometry field creation just after layer creation
+#' * `CurveGeometries`: `TRUE` if this dataset supports curve geometries
+#' * `Transactions`: `TRUE` if this dataset supports (efficient)
 #' transactions
-#' * `EmulatedTransactions`: `TRUE` if this datasource supports transactions
+#' * `EmulatedTransactions`: `TRUE` if this dataset supports transactions
 #' through emulation
-#' * `RandomLayerRead`: `TRUE` if this datasource has a dedicated
+#' * `RandomLayerRead`: `TRUE` if this dataset has a dedicated
 #' `GetNextFeature()` implementation, potentially returning features from
 #' layers in a non-sequential way
-#' * `RandomLayerWrite`: `TRUE` if this datasource supports calling
+#' * `RandomLayerWrite`: `TRUE` if this dataset supports calling
 #' `CreateFeature()` on layers in a non-sequential way
+#' * `AddFieldDomain`: `TRUE` if this dataset supports adding field domains
+#' (GDAL >= 3.3)
+#' * `DeleteFieldDomain`: `TRUE` if this dataset supports deleting field
+#' domains (GDAL >= 3.5)
+#' `UpdateFieldDomain`: `TRUE` if this dataset supports updating a an existing
+#' field domain by replacing its definition (GDAL >= 3.5)
 #'
-#' `ogr_ds_create()` creates a new vector datasource, optionally also creating
+#' `ogr_ds_create()` creates a new vector dataset, optionally also creating
 #' a layer, and optionally creating one or more fields on the layer.
 #' The attribute fields and geometry field(s) to create can be specified as a
 #' feature class definition (`layer_defn` as list, see [ogr_define]), or
@@ -61,8 +68,27 @@
 #' `ogr_ds_layer_names()` returns a character vector of layer names in a
 #' vector dataset, or `NULL` if no layers are found.
 #'
+#' `ogr_ds_field_domain_names()` returns a character vector with the names of
+#' all field domains stored in the dataset. Returns `NULL` and emits a warning
+#' if an error occurs or if the format does not support reading field domains.
+#' Returns a character vector of length 0 (`character(0)`) if the format
+#' supports field domains but none are present in the dataset.
+#' Requires GDAL >= 3.5. See [ogr_define] for information on field domains.
+#'
+#' `ogr_ds_add_field_domain()` adds a field domain to a vector dataset. Only a
+#' few drivers will support this operation, and some of them might only support
+#' it only for some types of field domains. GeoPackage and OpenFileGDB drivers
+#' support this operation. A dataset having at least some support should report
+#' the `AddFieldDomain` dataset capability (see `ogr_ds_test_cap()`above).
+#' Returns a logical value, `TRUE` indicating success. Requires GDAL >= 3.3.
+#'
+#' `ogr_ds_delete_field_domain()` deletes a field domain from a vector dataset
+#' if supported (see `ogr_ds_test_cap()`above for the `DeleteFieldDomain`
+#' dataset capability). Returns a logical value, `TRUE` indicating success.
+#' Requires GDAL >= 3.5.
+#'
 #' `ogr_layer_exists()` tests whether a layer can be accessed by name in a
-#' given vector dataset. Returns a logical scalar.
+#' given vector dataset. Returns a logical value
 #'
 #' `ogr_layer_test_cap()` tests whether a layer supports named capabilities,
 #' attempting to open the dataset with update access by default.
@@ -94,11 +120,11 @@
 #' `ogr_layer_rename()` renames a layer in a vector dataset. This operation is
 #' implemented only by layers that expose the `Rename` capability (see
 #' `ogr_layer_test_cap()` above). This operation will fail if a layer with the
-#' new name already exists. Returns a logical scalar, `TRUE` indicating success.
+#' new name already exists. Returns a logical value, `TRUE` indicating success.
 #' Requires GDAL >= 3.5.
 #'
 #' `ogr_layer_delete()` deletes an existing layer in a vector dataset.
-#' Returns a logical scalar, `TRUE` indicating success.
+#' Returns a logical value, `TRUE` indicating success.
 #'
 #' `ogr_field_index()` tests for existence of an attribute field by name.
 #' Returns the field index on the layer (0-based), or `-1` if the field does
@@ -106,21 +132,26 @@
 #'
 #' `ogr_field_create()` creates a new attribute field of specified data type in
 #' a given DSN/layer. Several optional field properties can be specified in
-#' addition to the type. Returns a logical scalar, `TRUE` indicating success.
+#' addition to the type. Returns a logical value, `TRUE` indicating success.
 #'
 #' `ogr_geom_field_create()` creates a new geometry field of specified type in
-#' a given DSN/layer. Returns a logical scalar, `TRUE` indicating success.
+#' a given DSN/layer. Returns a logical value, `TRUE` indicating success.
 #'
 #' `ogr_field_rename()` renames an existing field on a vector layer.
 #' Not all format drivers support this function. Some drivers may only support
 #' renaming a field while there are still no features in the layer.
 #' `AlterFieldDefn` is the relevant layer capability to check.
-#' Returns a logical scalar, `TRUE` indicating success.
+#' Returns a logical value, `TRUE` indicating success.
+#'
+#' `ogr_field_set_domain_name()` sets the field domain name for an existing
+#' attribute field on a vector layer. `AlterFieldDefn` layer capability is
+#' required. Returns a logical value, `TRUE` indicating success.
+#' Requires GDAL >= 3.3.
 #'
 #' `ogr_field_delete()` deletes an existing field on a vector layer.
 #' Not all format drivers support this function. Some drivers may only support
 #' deleting a field while there are still no features in the layer.
-#' Returns a logical scalar, `TRUE` indicating success.
+#' Returns a logical value, `TRUE` indicating success.
 #'
 #' `ogr_execute_sql()` executes an SQL statement against the data store.
 #' This function can be used to modify the schema or edit data using SQL
@@ -132,7 +163,7 @@
 #'
 #' @param dsn Character string. The vector data source name, e.g., a filename
 #' or database connection string.
-#' @param with_update Logical scalar. `TRUE` to request update access when
+#' @param with_update Logical value. `TRUE` to request update access when
 #' opening the dataset, or `FALSE` to open read-only.
 #' @param format GDAL short name of the vector format as character string.
 #' Examples of some common output formats include: `"GPKG"`, `"FlatGeobuf"`,
@@ -164,28 +195,30 @@
 #' Additional arguments in `ogr_field_create()` will be ignored if a `fld_defn`
 #' is given.
 #' @param fld_type Character string containing the name of a field data type
-#' (e.g., `OFTInteger`, `OFTReal`, `OFTString`).
+#' (e.g., `"OFTInteger"`, `"OFTInteger64"`, `"OFTReal"`, `"OFTString"`, ...).
 #' @param fld_subtype Character string containing the name of a field subtype.
-#' One of  `OFSTNone` (the default), `OFSTBoolean`, `OFSTInt16`, `OFSTFloat32`,
-#' `OFSTJSON`, `OFSTUUID`.
-#' @param fld_width Optional integer scalar specifying max number of characters.
-#' @param fld_precision Optional integer scalar specifying number of digits
+#' One of `"OFSTNone"` (the default), `"OFSTBoolean"`, `"OFSTInt16"`,
+#' `"OFSTFloat32"`, `"OFSTJSON"`, `"OFSTUUID"`.
+#' @param fld_width Optional integer value specifying max number of characters.
+#' @param fld_precision Optional integer value specifying number of digits
 #' after the decimal point.
-#' @param is_nullable Optional NOT NULL field constraint (logical scalar).
+#' @param is_nullable Optional NOT NULL field constraint (logical value).
 #' Defaults to `TRUE`.
-#' @param is_unique Optional UNIQUE constraint on the field (logical scalar).
+#' @param is_unique Optional UNIQUE constraint on the field (logical value).
 #' Defaults to `FALSE`.
 #' @param default_value Optional default value for the field as a character
 #' string.
-#' @param geom_fld_defn A geometry field definition as list
-#' (see [ogr_def_geom_field()]).
-#' Additional arguments in `ogr_geom_field_create()` will be ignored if a
-#' `geom_fld_defn` is given.
+#' @param domain_name Character string specifying the name of the field domain.
+#' @param geom_fld_defn A geometry field definition as list (see
+#' [ogr_def_geom_field()]). Additional arguments in `ogr_geom_field_create()`
+#' will be ignored if a `geom_fld_defn` is given.
 #' @param new_name Character string containing a new name to assign.
 #' @param dsco Optional character vector of format-specific creation options
 #' for `dsn` (`"NAME=VALUE"` pairs).
 #' @param lco Optional character vector of format-specific creation options
 #' for `layer` (`"NAME=VALUE"` pairs).
+#' @param fld_dom_defn A field domain definition, i.e., a list generated with
+#' [ogr_def_field_domain()].
 #' @param sql Character string containing an SQL statement (see Note).
 #' @param spatial_filter Either a numeric vector of length four containing a
 #' bounding box (xmin, ymin, xmax, ymax), or a character string containing a
@@ -193,9 +226,9 @@
 #' @param dialect Character string specifying the SQL dialect to use.
 #' The OGR SQL engine (`"OGRSQL"`) will be used by default if a value is not
 #' given. The `"SQLite"` dialect can also be used (see Note).
-#' @param overwrite Logical scalar. `TRUE` to overwrite `dsn` if it already
+#' @param overwrite Logical value. `TRUE` to overwrite `dsn` if it already
 #' exists when calling `ogr_ds_create()`. Default is `FALSE`.
-#' @param return_obj Logical scalar. If `TRUE`, an object of class
+#' @param return_obj Logical value. If `TRUE`, an object of class
 #' [`GDALVector`][GDALVector] open on the newly created layer will be
 #' returned. Defaults to `FALSE`. Must be used with either the `layer` or
 #' `layer_defn` arguments.
@@ -313,15 +346,24 @@
 #' \dontshow{deleteDataset(perims_shp)}
 #' @export
 ogr_ds_exists <- function(dsn, with_update = FALSE) {
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
 
+    if (is.null(with_update) || all(is.na(with_update)))
+        with_update <- FALSE
+    if (!is.logical(with_update) || length(with_update) > 1)
+        stop("'with_update' must be a single logical value", call. = FALSE)
+
     return(.ogr_ds_exists(dsn, with_update))
 }
 
 #' @name ogr_manage
 #' @export
 ogr_ds_format <- function(dsn) {
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
 
@@ -335,9 +377,16 @@ ogr_ds_format <- function(dsn) {
 #' @name ogr_manage
 #' @export
 ogr_ds_test_cap <- function(dsn, with_update = TRUE) {
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
 
+    if (is.null(with_update) || all(is.na(with_update)))
+        with_update <- FALSE
+    if (!is.logical(with_update) || length(with_update) > 1)
+        stop("'with_update' must be a single logical value", call. = FALSE)
+
     return(.ogr_ds_test_cap(dsn, with_update))
 }
 
@@ -349,15 +398,19 @@ ogr_ds_create <- function(format, dsn, layer = NULL, layer_defn = NULL,
                           overwrite = FALSE, return_obj = FALSE) {
 
     # format
+    if (missing(format) || is.null(format) || all(is.na(format)))
+        stop("'format' is required", call. = FALSE)
     if (!(is.character(format) && length(format) == 1))
         stop("'format' must be a length-1 character vector", call. = FALSE)
     # dsn
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
     if (vsi_stat(dsn) && !overwrite)
         stop("'dsn' exists but 'overwrite' is `FALSE`", call. = FALSE)
     # layer
-    if (is.null(layer))
+    if (missing(layer) || is.null(layer) || all(is.na(layer)))
         layer <- ""
     if (!(is.character(layer) && length(layer) == 1))
         stop("'layer' must be a length-1 character vector", call. = FALSE)
@@ -368,22 +421,22 @@ ogr_ds_create <- function(format, dsn, layer = NULL, layer_defn = NULL,
         }
     }
     # geom_type
-    if (is.null(geom_type))
-        geom_type <- ""
+    if (missing(geom_type) || is.null(geom_type) || all(is.na(geom_type)))
+        geom_type <- "UNKNOWN"
     if (!(is.character(geom_type) && length(geom_type) == 1))
         stop("'geom_type' must be a length-1 character vector", call. = FALSE)
     # srs
-    if (is.null(srs))
+    if (missing(srs) || is.null(srs) || all(is.na(srs)))
         srs <- ""
     if (!(is.character(srs) && length(srs) == 1))
         stop("'srs' must be a length-1 character vector", call. = FALSE)
     # fld_name
-    if (is.null(fld_name))
+    if (missing(fld_name) || is.null(fld_name) || all(is.na(fld_name)))
         fld_name <- ""
     if (!(is.character(fld_name) && length(fld_name) == 1))
         stop("'fld_name' must be a length-1 character vector", call. = FALSE)
     # fld_type
-    if (is.null(fld_type))
+    if (missing(fld_type) || is.null(fld_type) || all(is.na(fld_type)))
         fld_type <- ""
     if (!(is.character(fld_type) && length(fld_type) == 1))
         stop("'fld_type' must be a length-1 character vector", call. = FALSE)
@@ -398,15 +451,15 @@ ogr_ds_create <- function(format, dsn, layer = NULL, layer_defn = NULL,
             stop("'lco' must be a character vector", call. = FALSE)
     }
     # overwrite
-    if (is.null(overwrite))
-        stop("'overwrite' must be a logical value", call. = FALSE)
+    if (missing(overwrite) || is.null(overwrite) || all(is.na(overwrite)))
+        overwrite <- FALSE
     if (!is.logical(overwrite) || length(overwrite) > 1)
-        stop("'overwrite' must be a logical scalar", call. = FALSE)
+        stop("'overwrite' must be a single logical value", call. = FALSE)
     # return_obj
-    if (is.null(return_obj))
-        stop("'return_obj' must be a logical value", call. = FALSE)
+    if (missing(return_obj) || is.null(return_obj) || all(is.na(return_obj)))
+        return_obj <- FALSE
     if (!is.logical(return_obj) || length(return_obj) > 1)
-        stop("'return_obj' must be a logical scalar", call. = FALSE)
+        stop("'return_obj' must be a single logical value", call. = FALSE)
     if (return_obj && layer == "" && is.null(layer_defn)) {
         stop("'layer' or 'layer_defn' must be given with 'return_obj = TRUE'",
              call. = FALSE)
@@ -435,6 +488,8 @@ ogr_ds_create <- function(format, dsn, layer = NULL, layer_defn = NULL,
 #' @name ogr_manage
 #' @export
 ogr_ds_layer_count <- function(dsn) {
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
 
@@ -444,6 +499,8 @@ ogr_ds_layer_count <- function(dsn) {
 #' @name ogr_manage
 #' @export
 ogr_ds_layer_names <- function(dsn) {
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
 
@@ -452,9 +509,66 @@ ogr_ds_layer_names <- function(dsn) {
 
 #' @name ogr_manage
 #' @export
-ogr_layer_exists <- function(dsn, layer) {
+ogr_ds_field_domain_names <- function(dsn) {
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
+
+    return(.ogr_ds_field_domain_names(dsn))
+}
+
+#' @name ogr_manage
+#' @export
+ogr_ds_add_field_domain <- function(dsn, fld_dom_defn) {
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
+    if (!(is.character(dsn) && length(dsn) == 1))
+        stop("'dsn' must be a length-1 character vector", call. = FALSE)
+
+    if (!is.list(fld_dom_defn) || is.data.frame(fld_dom_defn))
+        stop("'fld_dom_defn' must be a list object", call. = FALSE)
+
+    if (ogr_ds_test_cap(dsn)$AddFieldDomain) {
+        return(.ogr_ds_add_field_domain(dsn, fld_dom_defn))
+    } else {
+        message("the dataset does not support AddFieldDomain")
+        return(FALSE)
+    }
+}
+
+#' @name ogr_manage
+#' @export
+ogr_ds_delete_field_domain <- function(dsn, domain_name) {
+    if (gdal_version_num() < gdal_compute_version(3, 5, 0))
+        stop("ogr_ds_delete_field_domain() requires GDAL >= 3.5", call. = FALSE)
+
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
+    if (!(is.character(dsn) && length(dsn) == 1))
+        stop("'dsn' must be a length-1 character vector", call. = FALSE)
+
+    if (!(is.character(domain_name) && length(domain_name) == 1))
+        stop("'domain_name' must be a length-1 character vector", call. = FALSE)
+
+    if (ogr_ds_test_cap(dsn)$DeleteFieldDomain) {
+        return(ogr_ds_delete_field_domain(dsn, domain_name))
+    } else {
+        message("the dataset does not support DeleteFieldDomain")
+        return(FALSE)
+    }
+}
+
+#' @name ogr_manage
+#' @export
+ogr_layer_exists <- function(dsn, layer) {
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
+    if (!(is.character(dsn) && length(dsn) == 1))
+        stop("'dsn' must be a length-1 character vector", call. = FALSE)
+
+    if (missing(layer) || is.null(layer) || all(is.na(layer)))
+        layer <- ""
     if (!(is.character(layer) && length(layer) == 1))
         stop("'layer' must be a length-1 character vector", call. = FALSE)
 
@@ -464,13 +578,21 @@ ogr_layer_exists <- function(dsn, layer) {
 #' @name ogr_manage
 #' @export
 ogr_layer_test_cap <- function(dsn, layer = NULL, with_update = TRUE) {
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
-    if (is.null(layer))
+
+    if (missing(layer) || is.null(layer) || all(is.na(layer)))
         layer <- ""
     if (!(is.character(layer) && length(layer) == 1))
         stop("'layer' must be a length-1 character vector", call. = FALSE)
 
+    if (is.null(with_update) || all(is.na(with_update)))
+        with_update <- TRUE
+    if (!is.logical(with_update) || length(with_update) > 1)
+        stop("'with_update' must be a single logical value", call. = FALSE)
+
     return(.ogr_layer_test_cap(dsn, layer, with_update))
 }
 
@@ -479,13 +601,18 @@ ogr_layer_test_cap <- function(dsn, layer = NULL, with_update = TRUE) {
 ogr_layer_create <- function(dsn, layer, layer_defn = NULL, geom_type = NULL,
                              srs = NULL, lco = NULL, return_obj = FALSE) {
 
-    # dsn / layer
+    # dsn
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
-    if (!(is.character(layer) && length(layer) == 1))
-        stop("'layer' must be a length-1 character vector", call. = FALSE)
     if (!ogr_ds_exists(dsn, with_update = TRUE))
         stop("'dsn' does not exist, or no update access", call. = FALSE)
+    # layer
+    if (missing(layer) || is.null(layer) || all(is.na(layer)) || layer == "")
+        stop("'layer' is required", call. = FALSE)
+    if (!(is.character(layer) && length(layer) == 1))
+        stop("'layer' must be a length-1 character vector", call. = FALSE)
     if (ogr_layer_exists(dsn, layer))
         stop("'layer' already exists", call. = FALSE)
     # layer_defn
@@ -522,12 +649,14 @@ ogr_layer_create <- function(dsn, layer, layer_defn = NULL, geom_type = NULL,
     }
 
     # geom_type
-    if (is.null(geom_type)) {
+    if (missing(geom_type) || is.null(geom_type) || all(is.na(geom_type))) {
         warning("geometry type not specified, using 'UNKNOWN'", call. = FALSE)
         geom_type <- "UNKNOWN"
     }
+    if (!(is.character(geom_type) && length(geom_type) == 1))
+        stop("'geom_type' must be a length-1 character vector", call. = FALSE)
     # srs
-    if (is.null(srs))
+    if (missing(srs) || is.null(srs) || all(is.na(srs)))
         srs <- ""
     if (!(is.character(srs) && length(srs) == 1))
         stop("'srs' must be a length-1 character vector", call. = FALSE)
@@ -537,10 +666,10 @@ ogr_layer_create <- function(dsn, layer, layer_defn = NULL, geom_type = NULL,
             stop("'lco' must be a character vector", call. = FALSE)
     }
     # return_obj
-    if (is.null(return_obj))
-        stop("'return_obj' must be a logical value", call. = FALSE)
+    if (missing(return_obj) || is.null(return_obj) || all(is.na(return_obj)))
+        return_obj <- FALSE
     if (!is.logical(return_obj) || length(return_obj) > 1)
-        stop("'return_obj' must be a logical scalar", call. = FALSE)
+        stop("'return_obj' must be a single logical value", call. = FALSE)
 
     # signature for ogr_layer_create() object factory
     lyr <- new(GDALVector, dsn, layer, layer_defn, geom_type, srs, lco, TRUE)
@@ -556,9 +685,12 @@ ogr_layer_create <- function(dsn, layer, layer_defn = NULL, geom_type = NULL,
 #' @name ogr_manage
 #' @export
 ogr_layer_field_names <- function(dsn, layer = NULL) {
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
-    if (is.null(layer))
+
+    if (missing(layer) || is.null(layer) || all(is.na(layer)))
         layer <- ""
     if (!(is.character(layer) && length(layer) == 1))
         stop("'layer' must be a length-1 character vector", call. = FALSE)
@@ -570,10 +702,24 @@ ogr_layer_field_names <- function(dsn, layer = NULL) {
 #' @name ogr_manage
 #' @export
 ogr_layer_rename <- function(dsn, layer, new_name) {
+    # dsn
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
+    if (!ogr_ds_exists(dsn, with_update = TRUE))
+        stop("'dsn' does not exist, or no update access", call. = FALSE)
+    # layer
+    if (missing(layer) || is.null(layer) || all(is.na(layer)) || layer == "")
+        stop("'layer' is required", call. = FALSE)
     if (!(is.character(layer) && length(layer) == 1))
         stop("'layer' must be a length-1 character vector", call. = FALSE)
+    # new_name
+    if (missing(new_name) || is.null(new_name) || all(is.na(new_name)) ||
+        new_name == "") {
+
+        stop("'new_name' is required", call. = FALSE)
+    }
     if (!(is.character(new_name) && length(new_name) == 1))
         stop("'new_name' must be a length-1 character vector", call. = FALSE)
 
@@ -583,8 +729,16 @@ ogr_layer_rename <- function(dsn, layer, new_name) {
 #' @name ogr_manage
 #' @export
 ogr_layer_delete <- function(dsn, layer) {
+    # dsn
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
+    if (!ogr_ds_exists(dsn, with_update = TRUE))
+        stop("'dsn' does not exist, or no update access", call. = FALSE)
+    # layer
+    if (missing(layer) || is.null(layer) || all(is.na(layer)) || layer == "")
+        stop("'layer' is required", call. = FALSE)
     if (!(is.character(layer) && length(layer) == 1))
         stop("'layer' must be a length-1 character vector", call. = FALSE)
 
@@ -594,10 +748,22 @@ ogr_layer_delete <- function(dsn, layer) {
 #' @name ogr_manage
 #' @export
 ogr_field_index <- function(dsn, layer, fld_name) {
+    # dsn
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
+    # layer
+    if (missing(layer) || is.null(layer) || all(is.na(layer)))
+        layer <- ""
     if (!(is.character(layer) && length(layer) == 1))
         stop("'layer' must be a length-1 character vector", call. = FALSE)
+    # fld_name
+    if (missing(fld_name) || is.null(fld_name) || all(is.na(fld_name)) ||
+        fld_name == "") {
+
+        stop("'fld_name' is required", call. = FALSE)
+    }
     if (!(is.character(fld_name) && length(fld_name) == 1))
         stop("'fld_name' must be a length-1 character vector", call. = FALSE)
 
@@ -614,23 +780,45 @@ ogr_field_create <- function(dsn, layer, fld_name,
                              fld_precision = 0L,
                              is_nullable = TRUE,
                              is_unique = FALSE,
-                             default_value = "") {
+                             default_value = "",
+                             domain_name = NULL) {
 
+    # dsn
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
-    if (!(is.character(layer) && length(layer) == 1))
-        stop("'layer' must be a length-1 character vector", call. = FALSE)
-    if (!(is.character(fld_name) && length(fld_name) == 1))
-        stop("'fld_name' must be a length-1 character vector", call. = FALSE)
-
     if (!ogr_ds_exists(dsn, with_update = TRUE))
         stop("'dsn' does not exist, or no update access", call. = FALSE)
+    # layer
+    if (missing(layer) || is.null(layer) || all(is.na(layer)))
+        layer <- ""
+    if (!(is.character(layer) && length(layer) == 1))
+        stop("'layer' must be a length-1 character vector", call. = FALSE)
     if (!ogr_layer_exists(dsn, layer))
         stop("'layer' does not exist", call. = FALSE)
+    # fld_name
+    if (missing(fld_name) || is.null(fld_name) || all(is.na(fld_name)) ||
+        fld_name == "") {
 
-    fld_idx <- ogr_field_index(dsn, layer, fld_name)
-    if (fld_idx >= 0)
-        stop("field '", fld_name, "' already exists", call. = FALSE)
+        stop("'fld_name' is required", call. = FALSE)
+    }
+    if (!(is.character(fld_name) && length(fld_name) == 1))
+        stop("'fld_name' must be a length-1 character vector", call. = FALSE)
+    # default_value
+    if (is.null(default_value) || all(is.na(default_value)))
+        default_value <- ""
+    else
+        default_value <- as.character(default_value)
+    if (!(length(default_value) == 1))
+        stop("'default_value' must be a length-1 character vector",
+             call. = FALSE)
+    # domain_name
+    if (is.null(domain_name) || all(is.na(domain_name)))
+        domain_name <- ""
+    if (!(is.character(domain_name) && length(domain_name) == 1))
+        stop("'domain_name' must be a length-1 character vector",
+             call. = FALSE)
 
     # use field definition if given
     if (!is.null(fld_defn)) {
@@ -671,11 +859,16 @@ ogr_field_create <- function(dsn, layer, fld_name,
             default_value <- fld_defn$default
         else
             default_value <- ""
+
+        if (!is.null(fld_defn$domain))
+            domain_name <- fld_defn$domain
+        else
+            domain_name <- ""
     }
 
     return(.ogr_field_create(dsn, layer, fld_name, fld_type, fld_subtype,
                              fld_width, fld_precision, is_nullable, is_unique,
-                             default_value))
+                             default_value, domain_name))
 }
 
 #' @name ogr_manage
@@ -686,31 +879,43 @@ ogr_geom_field_create <- function(dsn, layer, fld_name,
                                   srs = NULL,
                                   is_nullable = TRUE) {
 
+    # dsn
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
-    if (!(is.character(layer) && length(layer) == 1))
-        stop("'layer' must be a length-1 character vector", call. = FALSE)
-    if (!(is.character(fld_name) && length(fld_name) == 1))
-        stop("'fld_name' must be a length-1 character vector", call. = FALSE)
-
     if (!ogr_ds_exists(dsn, with_update = TRUE))
         stop("'dsn' does not exist, or no update access", call. = FALSE)
+    # layer
+    if (missing(layer) || is.null(layer) || all(is.na(layer)))
+        layer <- ""
+    if (!(is.character(layer) && length(layer) == 1))
+        stop("'layer' must be a length-1 character vector", call. = FALSE)
     if (!ogr_layer_exists(dsn, layer))
         stop("'layer' does not exist", call. = FALSE)
+    # fld_name
+    if (missing(fld_name) || is.null(fld_name) || all(is.na(fld_name)) ||
+        fld_name == "") {
 
-    fld_idx <- ogr_field_index(dsn, layer, fld_name)
-    if (fld_idx >= 0)
-        stop("field ", fld_name, " already exists", call. = FALSE)
-
-    if (is.null(geom_type))
+        stop("'fld_name' is required", call. = FALSE)
+    }
+    if (!(is.character(fld_name) && length(fld_name) == 1))
+        stop("'fld_name' must be a length-1 character vector", call. = FALSE)
+    # geom_type
+    if (is.null(geom_type) || all(is.na(geom_type)))
         geom_type <- "UNKNOWN"
     else if (!(is.character(geom_type) && length(geom_type) == 1))
         stop("'geom_type' must be a length-1 character vector", call. = FALSE)
-
-    if (is.null(srs))
+    # srs
+    if (is.null(srs) || all(is.na(srs)))
         srs <- ""
     else if (!(is.character(srs) && length(srs) == 1))
         stop("'srs' must be a length-1 character vector", call. = FALSE)
+    # is_nullable
+    if (missing(is_nullable) || is.null(is_nullable) || all(is.na(is_nullable)))
+        is_nullable <- TRUE
+    if (!is.logical(is_nullable) || length(is_nullable) > 1)
+        stop("'is_nullable' must be a single logical value", call. = FALSE)
 
     # use field definition if given
     if (!is.null(geom_fld_defn)) {
@@ -740,12 +945,34 @@ ogr_geom_field_create <- function(dsn, layer, fld_name,
 #' @name ogr_manage
 #' @export
 ogr_field_rename <- function(dsn, layer, fld_name, new_name) {
+    # dsn
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
+    if (!ogr_ds_exists(dsn, with_update = TRUE))
+        stop("'dsn' does not exist, or no update access", call. = FALSE)
+    # layer
+    if (missing(layer) || is.null(layer) || all(is.na(layer)))
+        layer <- ""
     if (!(is.character(layer) && length(layer) == 1))
         stop("'layer' must be a length-1 character vector", call. = FALSE)
+    if (!ogr_layer_exists(dsn, layer))
+        stop("'layer' does not exist", call. = FALSE)
+    # fld_name
+    if (missing(fld_name) || is.null(fld_name) || all(is.na(fld_name)) ||
+        fld_name == "") {
+
+        stop("'fld_name' is required", call. = FALSE)
+    }
     if (!(is.character(fld_name) && length(fld_name) == 1))
         stop("'fld_name' must be a length-1 character vector", call. = FALSE)
+    # new_name
+    if (missing(new_name) || is.null(new_name) || all(is.na(new_name)) ||
+        new_name == "") {
+
+        stop("'new_name' is required", call. = FALSE)
+    }
     if (!(is.character(new_name) && length(new_name) == 1))
         stop("'new_name' must be a length-1 character vector", call. = FALSE)
 
@@ -754,11 +981,61 @@ ogr_field_rename <- function(dsn, layer, fld_name, new_name) {
 
 #' @name ogr_manage
 #' @export
-ogr_field_delete <- function(dsn, layer, fld_name) {
+ogr_field_set_domain_name <- function(dsn, layer, fld_name, domain_name) {
+    # dsn
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
+    if (!ogr_ds_exists(dsn, with_update = TRUE))
+        stop("'dsn' does not exist, or no update access", call. = FALSE)
+    # layer
+    if (missing(layer) || is.null(layer) || all(is.na(layer)))
+        layer <- ""
     if (!(is.character(layer) && length(layer) == 1))
         stop("'layer' must be a length-1 character vector", call. = FALSE)
+    if (!ogr_layer_exists(dsn, layer))
+        stop("'layer' does not exist", call. = FALSE)
+    # fld_name
+    if (missing(fld_name) || is.null(fld_name) || all(is.na(fld_name)) ||
+        fld_name == "") {
+
+        stop("'fld_name' is required", call. = FALSE)
+    }
+    if (!(is.character(fld_name) && length(fld_name) == 1))
+        stop("'fld_name' must be a length-1 character vector", call. = FALSE)
+    # domain_name
+    if (missing(domain_name) || is.null(domain_name) || all(is.na(domain_name)))
+        stop("'domain_name' is required", call. = FALSE)
+    if (!(is.character(domain_name) && length(domain_name) == 1))
+        stop("'domain_name' must be a length-1 character vector", call. = FALSE)
+
+    return(.ogr_field_set_domain_name(dsn, layer, fld_name, domain_name))
+}
+
+#' @name ogr_manage
+#' @export
+ogr_field_delete <- function(dsn, layer, fld_name) {
+    # dsn
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
+    if (!(is.character(dsn) && length(dsn) == 1))
+        stop("'dsn' must be a length-1 character vector", call. = FALSE)
+    if (!ogr_ds_exists(dsn, with_update = TRUE))
+        stop("'dsn' does not exist, or no update access", call. = FALSE)
+    # layer
+    if (missing(layer) || is.null(layer) || all(is.na(layer)))
+        layer <- ""
+    if (!(is.character(layer) && length(layer) == 1))
+        stop("'layer' must be a length-1 character vector", call. = FALSE)
+    if (!ogr_layer_exists(dsn, layer))
+        stop("'layer' does not exist", call. = FALSE)
+    # fld_name
+    if (missing(fld_name) || is.null(fld_name) || all(is.na(fld_name)) ||
+        fld_name == "") {
+
+        stop("'fld_name' is required", call. = FALSE)
+    }
     if (!(is.character(fld_name) && length(fld_name) == 1))
         stop("'fld_name' must be a length-1 character vector", call. = FALSE)
 
@@ -768,11 +1045,17 @@ ogr_field_delete <- function(dsn, layer, fld_name) {
 #' @name ogr_manage
 #' @export
 ogr_execute_sql <- function(dsn, sql, spatial_filter = NULL, dialect = NULL) {
+    # dsn
+    if (missing(dsn) || is.null(dsn) || all(is.na(dsn)))
+        stop("'dsn' is required", call. = FALSE)
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
+    # sql
+    if (missing(sql) || is.null(sql) || all(is.na(sql)) || sql == "")
+        stop("'sql' is required", call. = FALSE)
     if (!(is.character(sql) && length(sql) == 1))
         stop("'sql' must be a length-1 character vector", call. = FALSE)
-
+    # spatial_filter
     if (!is.null(spatial_filter)) {
         if (is.numeric(spatial_filter) && length(spatial_filter) == 4)
             spatial_filter <- bbox_to_wkt(spatial_filter)
@@ -782,7 +1065,7 @@ ogr_execute_sql <- function(dsn, sql, spatial_filter = NULL, dialect = NULL) {
     } else {
         spatial_filter <- ""
     }
-
+    # dialect
     if (!is.null(dialect)) {
         if (!(is.character(dialect) && length(dialect) == 1))
             stop("'dialect' must be a length-1 character vector", call. = FALSE)
